/****************************************************************************
 * Constant and miscellanious functions definition                          *
 * Copyright (C) 2009 Mario J. Juha and Dr. Daniel Simkins                  *
 * Written by Mario J. Juha <mariojesusjuha@gmail.com>                      *
 *                                                                          * 
 *           University of South Florida                                    *
 *        Civil and Environmental Engineering                               *
 *                 Tampa - Florida                                          *
 * Record of revisions:                                                     *
 *   Date      Programmer   Description of change                           *
 * ========    ==========   =====================                           *
 *  unknown    Dr. Simkins    Original Code                                 *
 * 07/22/09    Mario Juha     Change REAL to double                         *
 * 06/01/10    Mario Juha     Added ran2 function                           *
 *                                                                          *
 * Description:                                                             *  
 * ------------                                                             *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License, or      *
 *   (at your option) any later version.                                    *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 ***************************************************************************/

#ifndef _CONST_NUM_H_
#define _CONST_NUM_H_

const double Pi = 3.1415926535897932384 ;
const double TwoPi = 2.0*Pi ;
const double real_eps = 2.22044604925e-16 ; 
const double bigreal = 1.0e99 ; 
const double zero = 0.0 ;
const double one = 1.0 ;
const double two = 2.0 ;
const double three = 3.0 ;
const double four = 4.0 ;
const double five = 5.0 ;
const double six = 6.0 ;
const double seven = 7.0 ;
const double eight = 8.0 ;
const double nine = 9.0 ;
const double ten = 10.0 ;
const double half = 0.5 ;
const double threehalfs = 1.5 ;
const double third = 1.0 / 3.0 ;
const double twothirds = 2.0/3.0 ;
const double fourth = 0.25 ;
const double fifth = 0.2 ;
const double sixth = 1.0 / 6.0 ;
const double seventh = 1.0 / 7.0 ;
const double eighth = 0.125 ;
const double ninth = 1.0 / 9.0 ;
const double tenth = 0.1 ;
const double ZeroTol = 1.0e-12 ; // is it a good value?

#define SQ(a) ( (a)*(a) )  // square root of 'a'

// to be used inside ran2(idum) function
#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

// Long period (> 2 x 10^18) random number generator of L'Ecuyer with
// Bays-Durham shuffle and added safeguards. Returns a uniform random
// deviate between 0.0 and 1.0 (exclusive of the endpoint
// values). Call with idum a negative integer to initialize;
// thereafter, do not alter idum between successive deviates in a
// sequence. RNMX should approximate the largest floating value that
// is less than 1. 
//
// W. Press, S. Teukolsky, W. Vetterling and B. Flannery. Numerical
// Recipies in C. The Art of Scientific Computing. Second
// Edition. Cambrige University Press. 2002. Page 282.
extern float ran2(long &idum) ;

extern void error(const char *routine, const char *message) ;
extern void warning(const char *routine, const char *message) ;

#endif
